@@ -62,6 +62,7 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->stdio_fd = -1;
 	d->status = BURN_DISC_UNREADY;
 	d->do_stream_recording = 0;
+        d->stream_recording_start= 0;
 	return 1;
 }
 
@@ -196,6 +197,31 @@ unsigned int burn_drive_count(void)
 }
 
 
+/* ts A80801 */
+int burn_drive_is_listed(char *path, struct burn_drive **found, int flag)
+{
+	int i, ret;
+	char drive_adr[BURN_DRIVE_ADR_LEN], off_adr[BURN_DRIVE_ADR_LEN];
+
+	ret = burn_drive_convert_fs_adr(path, off_adr);
+	if (ret <= 0)
+		strcpy(off_adr, path);
+	for (i = 0; i <= drivetop; i++) {
+		if (drive_array[i].global_index < 0)
+	continue;
+		ret = burn_drive_d_get_adr(&(drive_array[i]), drive_adr);
+		if (ret <= 0)
+	continue;
+		if(strcmp(off_adr, drive_adr) == 0) {
+			if (found != NULL)
+				*found= &(drive_array[i]);
+			return 1;
+		}
+	}
+	return 0;
+}
+
+
 /* ts A61125 : media status aspects of burn_drive_grab() */
 int burn_drive_inquire_media(struct burn_drive *d)
 {
@@ -599,7 +625,7 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 	int ret, buf_secs, err, i, stages = 1, pbase, pfill, pseudo_sector;
 	off_t num_bufs;
 	char msg[80];
-	struct buffer buf;
+	struct buffer buf, *buf_mem = d->buffer;
 
 	/* reset the progress */
 	d->progress.session = 0;
@@ -681,7 +707,7 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 ex:;
 	d->progress.sector = 0x10000;
 	d->busy = BURN_DRIVE_IDLE;
-	d->buffer = NULL;
+	d->buffer = buf_mem;
 }
 
 
@@ -767,6 +793,18 @@ enum burn_drive_status burn_drive_get_status(struct burn_drive *d,
 	return d->busy;
 }
 
+int burn_drive_set_stream_recording(struct burn_drive *d, int recmode,
+                                    int start, int flag)
+{
+
+	if (recmode == 1)
+		d->do_stream_recording = 1;
+	else if (recmode == -1)
+		d->do_stream_recording = 0;
+	d->stream_recording_start = start;
+	return(1);
+}
+
 void burn_drive_cancel(struct burn_drive *d)
 {
 	pthread_mutex_lock(&d->access_lock);
@@ -1166,6 +1204,20 @@ int burn_drive_is_banned(char *device_address)
 }
 
 
+/* ts A80731 */
+int burn_drive_whitelist_count(void)
+{
+	return enumeration_whitelist_top + 1;
+}
+
+char *burn_drive_whitelist_item(int idx, int flag)
+{
+	if (idx < 0 || idx > enumeration_whitelist_top)
+		return NULL;
+	return enumeration_whitelist[idx];
+}
+
+
 /* ts A70924 */
 int burn_drive__fd_from_special_adr(char *adr)
 {
@@ -1705,10 +1757,18 @@ int burn_abort(int patience,
 			occup = burn_drive_is_occupied(&(drive_array[i]));
 			if(occup == -2)
 		continue;
+
+#ifdef NIX
+			/* <<< this causes a race condition with drive usage
+				and drive disposal
+			*/
 			if(drive_array[i].drive_role != 1) {
 				drive_array[i].busy = BURN_DRIVE_IDLE;
 				burn_drive_forget(&(drive_array[i]), 1);
-			} else if(occup <= 10) {
+			} else
+#endif /* NIX */
+
+			if(occup <= 10) {
 				burn_drive_forget(&(drive_array[i]), 1);
 			} else if(occup <= 100) {
 				if(first_round)
@@ -2207,8 +2267,9 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		o->might_do_sao = 4;
 		o->might_do_tao = 2;
 		o->advised_write_mode = BURN_WRITE_TAO;
-	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b) {
-		/* DVD+R , DVD+R/DL */
+	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b ||
+		   d->current_profile == 0x41) {
+		/* DVD+R , DVD+R/DL , BD-R SRM */
 		o->multi_session = o->multi_track = 1;
 		o->might_do_tao = 2;
 		o->might_do_sao = 1;
@@ -2467,3 +2528,10 @@ int burn_drive_set_media_capacity_remaining(struct burn_drive *d, off_t value)
 	return 1;
 }
 
+/* ts A81215 : API */
+int burn_get_read_capacity(struct burn_drive *d, int *capacity, int flag)
+{
+	*capacity = d->media_read_capacity;
+	return (d->media_read_capacity != 0x7fffffff);
+}
+
