@@ -644,12 +644,193 @@ failed:;
 }
 
 
+#define Libburn_drive_new_deaL 1
+
+#ifdef Libburn_drive_new_deaL
+
+/* ts A80731 */
+static int is_ata_drive(char *fname)
+{
+	int fd;
+	struct hd_driveid tm;
+
+	fd = sg_open_drive_fd(fname, 1);
+	if (fd == -1) {
+		if (linux_ata_enumerate_verbous)
+			fprintf(stderr,"open failed, errno=%d  '%s'\n",
+				errno, strerror(errno));
+		return 0;
+	}
+
+	memset(&tm, 0, sizeof(tm));
+	ioctl(fd, HDIO_GET_IDENTITY, &tm);
+
+		/* not atapi */
+	if (!(tm.config & 0x8000) || (tm.config & 0x4000)) {
+		if (linux_ata_enumerate_verbous)
+			fprintf(stderr, "not marked as ATAPI\n");
+		sg_close_drive_fd(fname, -1, &fd, 0);
+		return 0;
+	}
+
+	/* if SG_IO fails on an atapi device, we should stop trying to 
+	   use hd* devices */
+	if (sgio_test(fd) == -1) {
+		if (linux_ata_enumerate_verbous)
+		  fprintf(stderr,
+			 "FATAL: sgio_test() failed: errno=%d  '%s'\n",
+			 errno, strerror(errno));
+		sg_close_drive_fd(fname, -1, &fd, 0);
+		return 0;
+	}
+	if (sg_close_drive_fd(fname, -1, &fd, 1) <= 0) {
+		if (linux_ata_enumerate_verbous)
+			fprintf(stderr,
+				"cannot close properly, errno=%d  '%s'\n",
+				errno, strerror(errno));
+		return 0;
+	}
+	return 1;
+}
+
+
+static int is_scsi_drive(char *fname, int *bus_no, int *host_no,
+			 int *channel_no, int *target_no, int *lun_no)
+{
+	int fd, sid_ret = 0, ret;
+	struct sg_scsi_id sid;
+	int sibling_fds[BURN_OS_SG_MAX_SIBLINGS], sibling_count= 0;
+	char sibling_fnames[BURN_OS_SG_MAX_SIBLINGS][BURN_OS_SG_MAX_NAMELEN];
+
+	fd = sg_open_drive_fd(fname, 1);
+	if (fd == -1) {
+		if (linux_sg_enumerate_debug)
+			fprintf(stderr, "open failed, errno=%d  '%s'\n",
+				errno, strerror(errno));
+		return 0;
+	}
+
+	sid_ret = ioctl(fd, SG_GET_SCSI_ID, &sid);
+	if (sid_ret == -1) {
+		sid.scsi_id = -1; /* mark SCSI address as invalid */
+		if(linux_sg_enumerate_debug) 
+			fprintf(stderr,
+			"ioctl(SG_GET_SCSI_ID) failed, errno=%d  '%s' , ",
+			errno, strerror(errno));
+
+		if (sgio_test(fd) == -1) {
+			if (linux_sg_enumerate_debug)
+		  		fprintf(stderr,
+				 "FATAL: sgio_test() failed: errno=%d  '%s'",
+				errno, strerror(errno));
+
+			sg_close_drive_fd(fname, -1, &fd, 0);
+			return 0;
+		}
+
+#ifdef CDROM_DRIVE_STATUS
+		/* http://developer.osdl.org/dev/robustmutexes/
+			  src/fusyn.hg/Documentation/ioctl/cdrom.txt */
+		sid_ret = ioctl(fd, CDROM_DRIVE_STATUS, 0);
+		if(linux_sg_enumerate_debug)
+			  fprintf(stderr,
+				"ioctl(CDROM_DRIVE_STATUS) = %d , ",
+				sid_ret);
+		if (sid_ret != -1 && sid_ret != CDS_NO_INFO)
+			sid.scsi_type = TYPE_ROM;
+		else
+			sid_ret = -1;
+#endif /* CDROM_DRIVE_STATUS */
+
+	}
+
+#ifdef SCSI_IOCTL_GET_BUS_NUMBER
+	/* Hearsay A61005 */
+	if (ioctl(fd, SCSI_IOCTL_GET_BUS_NUMBER, bus_no) == -1)
+		*bus_no = -1;
+#endif
+
+	if (sg_close_drive_fd(fname, -1, &fd, 
+					sid.scsi_type == TYPE_ROM ) <= 0) {
+		if (linux_sg_enumerate_debug)
+			fprintf(stderr,
+				"cannot close properly, errno=%d  '%s'\n",
+				errno, strerror(errno)); 
+		return 0;
+	}
+	if ( (sid_ret == -1 || sid.scsi_type != TYPE_ROM)
+	     && !linux_sg_accept_any_type) {
+		if (linux_sg_enumerate_debug)
+			fprintf(stderr, "sid.scsi_type = %d (!= TYPE_ROM)\n",
+				sid.scsi_type); 
+		return 0;
+	}
+
+	if (sid_ret == -1 || sid.scsi_id < 0) {
+		/* ts A61211 : employ a more general ioctl */
+		ret = sg_obtain_scsi_adr(fname, bus_no, host_no,
+					   channel_no, target_no, lun_no);
+		if (ret>0) {
+			sid.host_no = *host_no;
+			sid.channel = *channel_no;
+			sid.scsi_id = *target_no;
+			sid.lun = *lun_no;
+		} else {
+			if (linux_sg_enumerate_debug)
+				fprintf(stderr,
+					"sg_obtain_scsi_adr() failed\n");
+			return 0;
+		}
+	}
+
+	/* ts A60927 : trying to do locking with growisofs */
+	if(burn_sg_open_o_excl>1) {
+		ret = sg_open_scsi_siblings(
+				fname, -1, sibling_fds, sibling_fnames,
+				&sibling_count,
+				sid.host_no, sid.channel,
+				sid.scsi_id, sid.lun);
+		if (ret<=0) {
+			if (linux_sg_enumerate_debug)
+				fprintf(stderr, "cannot lock siblings\n"); 
+				sg_handle_busy_device(fname, 0);
+			return 0;
+		}
+		/* the final occupation will be done in sg_grab() */
+		sg_release_siblings(sibling_fds, sibling_fnames,
+							&sibling_count);
+	}
+#ifdef SCSI_IOCTL_GET_BUS_NUMBER
+	if(*bus_no == -1)
+		*bus_no = 1000 * (sid.host_no + 1) + sid.channel;
+#else
+	*bus_no = sid.host_no;
+#endif
+	*host_no= sid.host_no;
+	*channel_no= sid.channel;
+	*target_no= sid.scsi_id;
+	*lun_no= sid.lun;
+	return 1;
+}	
+
+#endif /* Libburn_drive_new_deaL */
+
+
 /** Speciality of Linux: detect non-SCSI ATAPI (EIDE) which will from
    then on used used via generic SCSI as is done with (emulated) SCSI drives */ 
 static void ata_enumerate(void)
 {
+
+#ifdef Libburn_drive_new_deaL
+
+	int ret;
+
+#else 
 	struct hd_driveid tm;
-	int i, fd;
+	int fd;
+#endif
+
+	int i;
 	char fname[10];
 
 	if (linux_ata_enumerate_verbous)
@@ -670,6 +851,17 @@ static void ata_enumerate(void)
 				fprintf(stderr, "not in whitelist\n");
 	continue;
 		}
+
+#ifdef Libburn_drive_new_deaL
+
+		ret = is_ata_drive(fname);
+		if (ret < 0)
+	break;
+		if (ret == 0)
+	continue;
+
+#else /* Libburn_drive_new_deaL */
+
 		fd = sg_open_drive_fd(fname, 1);
 		if (fd == -1) {
 			if (linux_ata_enumerate_verbous)
@@ -706,6 +898,10 @@ static void ata_enumerate(void)
 				errno, strerror(errno));
 	continue;
 		}
+
+#endif /* Libburn_drive_new_deaL */
+
+
 		if (linux_ata_enumerate_verbous)
 		  fprintf(stderr, "accepting as drive without SCSI address\n");
 		enumerate_common(fname, -1, -1, -1, -1, -1);
@@ -716,12 +912,18 @@ static void ata_enumerate(void)
 /** Detects (probably emulated) SCSI drives */
 static void sg_enumerate(void)
 {
+#ifdef Libburn_drive_new_deaL
+
+#else 
 	struct sg_scsi_id sid;
-	int i, fd, sibling_fds[BURN_OS_SG_MAX_SIBLINGS], sibling_count= 0, ret;
+	int fd, sibling_fds[BURN_OS_SG_MAX_SIBLINGS], sibling_count= 0;
+	char sibling_fnames[BURN_OS_SG_MAX_SIBLINGS][BURN_OS_SG_MAX_NAMELEN];
 	int sid_ret = 0;
+#endif
+
+	int i, ret;
 	int bus_no= -1, host_no= -1, channel_no= -1, target_no= -1, lun_no= -1;
 	char fname[17];
-	char sibling_fnames[BURN_OS_SG_MAX_SIBLINGS][BURN_OS_SG_MAX_NAMELEN];
 
         sg_select_device_family();
 
@@ -748,6 +950,23 @@ static void sg_enumerate(void)
 	continue;
 		}
 
+
+#ifdef Libburn_drive_new_deaL
+
+		ret = is_scsi_drive(fname, &bus_no, &host_no, &channel_no,
+							&target_no, &lun_no);
+		if (ret < 0)
+	break;
+		if (ret == 0)
+	continue;
+		if (linux_sg_enumerate_debug)
+		  fprintf(stderr, "accepting as SCSI %d,%d,%d,%d bus=%d\n",
+			  host_no, channel_no, target_no, lun_no, bus_no);
+		enumerate_common(fname, bus_no, host_no, channel_no, 
+				target_no, lun_no);
+
+#else /* Libburn_drive_new_deaL */
+
 		/* ts A60927 */
 		fd = sg_open_drive_fd(fname, 1);
 		if (fd == -1) {
@@ -857,16 +1076,223 @@ static void sg_enumerate(void)
 #else
 		bus_no = sid.host_no;
 #endif
+
 		if (linux_sg_enumerate_debug)
 		  fprintf(stderr, "accepting as SCSI %d,%d,%d,%d bus=%d\n",
 			  sid.host_no, sid.channel, sid.scsi_id, sid.lun,
 			  bus_no);
 		enumerate_common(fname, bus_no, sid.host_no, sid.channel, 
 				 sid.scsi_id, sid.lun);
+#endif /* Libburn_drive_new_deaL */
+
 	}
 }
 
 
+#ifdef Libburn_drive_new_deaL
+
+
+/* ts A80805 : eventually produce the other official name of a device file */
+static int fname_other_name(char *fname, char other_name[80], int flag)
+{
+	if(strncmp(fname, "/dev/sr", 7) == 0 &&
+	   (fname[7] >= '0' && fname[7] <= '9') &&
+           (fname[8] == 0 ||
+	    (fname[8] >= '0' && fname[8] <= '9' && fname[9] == 0))) {
+		sprintf(other_name, "/dev/scd%s", fname + 7);
+		return 1;
+	}
+	if(strncmp(fname, "/dev/scd", 8) == 0 &&
+	   (fname[8] >= '0' && fname[8] <= '9') &&
+           (fname[9] == 0 ||
+	    (fname[9] >= '0' && fname[9] <= '9' && fname[10] == 0))) {
+		sprintf(other_name, "/dev/sr%s", fname + 8);
+		return 1;
+	}
+	return 0;
+}
+
+
+/* ts A80805 */
+static int fname_drive_is_listed(char *fname, int flag)
+{
+	char other_fname[80];
+
+	if (burn_drive_is_listed(fname, NULL, 0))
+		return 1;
+	if (fname_other_name(fname, other_fname, 0) > 0)
+		if (burn_drive_is_listed(other_fname, NULL, 0))
+			return 2;
+	return 0;
+}
+
+
+/* ts A80731 : Directly open the given address.
+   @param flag bit0= do not complain about missing file
+               bit1= do not check whether drive is already listed
+*/
+static int fname_enumerate(char *fname, int flag)
+{
+	int is_ata= 0, is_scsi= 0;
+	int bus_no= -1, host_no= -1, channel_no= -1, target_no= -1, lun_no= -1;
+	char msg[BURN_DRIVE_ADR_LEN + 80];
+	struct stat stbuf;
+
+	if (!(flag & 2))
+		if (fname_drive_is_listed(fname, 0))
+			return 2;
+	if (stat(fname, &stbuf) == -1) {
+		sprintf(msg, "File object '%s' not found", fname);
+		if (!(flag & 1))
+			libdax_msgs_submit(libdax_messenger, -1, 0x0002000b,
+			  LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			  msg, 0, 0);
+		return -1;
+	}
+	
+	is_ata = is_ata_drive(fname);
+	if (is_ata < 0)
+		return -1;
+	if (!is_ata)
+		is_scsi = is_scsi_drive(fname, &bus_no, &host_no, &channel_no,
+					&target_no, &lun_no);
+	if (is_scsi < 0)
+		return -1;
+	if (is_ata == 0 && is_scsi == 0)
+		return 0;
+
+	if (linux_sg_enumerate_debug)
+		  fprintf(stderr,
+			"(single) accepting as SCSI %d,%d,%d,%d bus=%d\n",
+			host_no, channel_no, target_no, lun_no, bus_no);
+	enumerate_common(fname, bus_no, host_no, channel_no, 
+				target_no, lun_no);
+	return 1;
+}
+
+
+/* ts A80731 : Directly open the given address from a single-item whitlist */
+static int single_enumerate(int flag)
+{
+	int ret, wl_count;
+	char *fname, msg[BURN_DRIVE_ADR_LEN + 80];
+
+        wl_count= burn_drive_whitelist_count();
+	if (wl_count != 1)
+		return 0;
+	fname= burn_drive_whitelist_item(0, 0);
+	if (fname == NULL)
+		return 0;
+	ret = fname_enumerate(fname, 2);
+	if (ret <= 0) {
+		sprintf(msg, "Cannot access '%s' as SG_IO CDROM drive", fname);
+		libdax_msgs_submit(libdax_messenger, -1, 0x0002000a,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
+		ret = -1;
+	}
+	return ret;
+}
+
+
+/* ts A80801 : looking up drives listed in /proc/sys/dev/cdrom/info line like:
+                drive name:             sr1     hdc     hda     sr0
+   @parm flag bit0= release list memory and exit
+*/
+static int proc_sys_dev_cdrom_info(char ***list, int *count, int flag)
+{
+	FILE *fp;
+	char line[1024], fname[1024 + 5], *cpt, *retpt, *list_data;
+	int maxl= 0, pass, i;
+
+	if (*list != NULL) {
+		if ((*list)[0] != NULL)
+			free((*list)[0]);
+		free(*list);
+		*list = NULL;
+		*count = 0;
+	}
+	if (flag & 1) 
+		return 1;
+
+	*count = 0;
+	sg_evaluate_kernel();
+	if (sg_kernel_age < 2) /* addresses are not suitable for kernel 2.4 */
+		return 1;
+	fp = fopen("/proc/sys/dev/cdrom/info", "r");
+	if (fp == NULL)
+		return 0;
+	while (1) {
+		retpt = fgets(line, sizeof(line), fp);
+		if (retpt == NULL)
+	break;
+		if(strncmp(line, "drive name:", 11) == 0)
+	break;
+	}
+	fclose(fp);
+	if (retpt == NULL)
+		return 0;
+	strcpy(fname, "/dev/");
+	for(pass = 0; pass < 2; pass++) {
+		*count = 0;
+		cpt = line + 11;
+		while (*cpt != 0) {
+			for(; *cpt == ' ' || *cpt == '\t'; cpt++);
+			if (*cpt == 0 || *cpt == '\n')
+		break;
+			sscanf(cpt, "%s", fname + 5);
+			if (strlen(fname) > maxl)
+				maxl = strlen(fname);
+			if (pass == 1)
+				strcpy((*list)[*count], fname);
+			(*count)++;
+			for(cpt++; *cpt != ' ' && *cpt != '\t'
+					 && *cpt != 0 && *cpt != '\n'; cpt++);
+		}
+		if (pass == 0) {
+			list_data = calloc(*count + 1, maxl+1);
+			*list = calloc(*count + 1, sizeof(char *));
+			if(list_data == NULL || *list == NULL) {
+				libdax_msgs_submit(libdax_messenger, -1,
+				0x00000003,
+				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+				"Out of virtual memory", 0, 0);
+				if (list_data != NULL)
+					free(list_data);
+				if (*list != NULL)
+					free((char *) *list);
+				return -1;
+			}
+			for (i = 0; i <= *count; i++)
+				(*list)[i] = list_data + i * (maxl + 1);
+		}
+	}
+	return 1;
+}
+
+
+static int add_proc_info_drives(int flag)
+{
+	int ret, list_count, count = 0, i;
+	char **list= NULL;
+
+	ret = proc_sys_dev_cdrom_info(&list, &list_count, 0);
+	if (ret <= 0)
+		return ret;
+	for (i = 0; i < list_count; i++) {
+		if (burn_drive_is_banned(list[i]))
+	continue;
+		ret = fname_enumerate(list[i], 1);
+		if (ret == 1)
+			count++;
+	}
+	proc_sys_dev_cdrom_info(&list, &list_count, 1); /* free memory */
+	return 1 + count;
+}
+
+#endif /* Libburn_drive_new_deaL */
+
+
 /* ts A61115 */
 /* ----------------------------------------------------------------------- */
 /* PORTING: Private functions which contain publicly needed functionality. */
@@ -951,10 +1377,14 @@ int sg_give_next_adr(burn_drive_enumerator_t *idx,
 {
 	/* os-linux.h : typedef int burn_drive_enumerator_t; */
 	static int sg_limit = 32, ata_limit = 26;
-	int baseno = 0;
+	int baseno = 0, i;
+	char other_name[80];
 
-	if (initialize == -1)
+	if (initialize == -1) {
+		proc_sys_dev_cdrom_info(&(idx->info_list), &(idx->info_count),
+					1);
 		return 0;
+	}
 
         sg_select_device_family();
 	if (linux_sg_device_family[0] == 0)
@@ -962,30 +1392,56 @@ int sg_give_next_adr(burn_drive_enumerator_t *idx,
 	if (linux_ata_device_family[0] == 0)
 		ata_limit = 0;
 
-	if (initialize  == 1)
-		*idx = -1;
-	(*idx)++;
-	if (*idx >= sg_limit)
+	if (initialize  == 1) {
+		idx->pos = -1;
+		idx->info_count= 0;
+		idx->info_list= NULL;
+		proc_sys_dev_cdrom_info(&(idx->info_list), &(idx->info_count),
+					0);
+	}
+	(idx->pos)++;
+	if (idx->pos >= sg_limit)
 		goto next_ata;
-	if (adr_size < 10)
+	if (adr_size < 11)
 		return -1;
-	sprintf(adr, linux_sg_device_family, *idx);
+	sprintf(adr, linux_sg_device_family, idx->pos);
 
-	/* ts A80702 */
 	sg_exchange_scd_for_sr(adr, 0);
+	goto return_1_pre_proc;
 
-	return 1;
 next_ata:;
 	baseno += sg_limit;
-	if (*idx - baseno >= ata_limit)
-		goto next_nothing;
+	if (idx->pos - baseno >= ata_limit)
+		goto next_proc_info;
 	if (adr_size < 9)
 		return -1;
-	sprintf(adr, linux_ata_device_family, 'a' + (*idx - baseno));
-	return 1;
-next_nothing:;
+	sprintf(adr, linux_ata_device_family, 'a' + (idx->pos - baseno));
+	goto return_1_pre_proc;
+
+next_proc_info:;
 	baseno += ata_limit;
+	for (i = 0; i < idx->info_count; i++) {
+		if ((idx->info_list)[i][0] == 0)
+	continue;
+		if (baseno == idx->pos) {
+			if (adr_size < strlen((idx->info_list)[i]) + 1)
+				return -1;
+			strcpy(adr, (idx->info_list)[i]);
+			return 1;
+		}
+		baseno++;
+	}
 	return 0;
+
+return_1_pre_proc:;
+	for (i = 0; i < idx->info_count; i++) {
+		if (strcmp((idx->info_list)[i], adr) == 0)
+			(idx->info_list)[i][0] = 0;
+	        if (fname_other_name(adr, other_name, 0) > 0)
+			if (strcmp((idx->info_list)[i], other_name) == 0)
+				(idx->info_list)[i][0] = 0;
+	}
+	return 1;
 }
 
 
@@ -1007,8 +1463,27 @@ next_nothing:;
 /* ts A61115: replacing call to sg-implementation internals from drive.c */
 int scsi_enumerate_drives(void)
 {
+
+#ifdef Libburn_drive_new_deaL
+	int ret;
+#endif
+
+#ifdef Libburn_drive_new_deaL
+	/* Direct examination of eventually single whitelisted name */
+	ret = single_enumerate(0);
+	if (ret < 0)
+		return -1;
+	if (ret > 0)
+		return 1;
+#endif /* Libburn_drive_new_deaL */
+
 	sg_enumerate();
 	ata_enumerate();
+
+#ifdef Libburn_drive_new_deaL
+	add_proc_info_drives(0);
+#endif /* Libburn_drive_new_deaL */
+
 	return 1;
 }
 
@@ -1460,7 +1935,8 @@ int sg_is_enumerable_adr(char *adr)
 			return 1;
 		}
 	}
-	sg_give_next_adr(&idx, fname, sizeof(fname), -1);
+	if (first == 0)
+		sg_give_next_adr(&idx, fname, sizeof(fname), -1);
 	return(0);
 }
 
