@@ -104,6 +104,30 @@ int burn_disc_add_session(struct burn_disc *d, struct burn_session *s,
 	return 1;
 }
 
+
+/* ts A81202: this function was in the API but not implemented.
+*/
+int burn_disc_remove_session(struct burn_disc *d, struct burn_session *s)
+{
+	int i, skip = 0;
+
+	if (d->session == NULL)
+		return 0;
+	for (i = 0; i < d->sessions; i++) {
+		if (s == d->session[i]) {
+			skip++;
+	continue;
+		}
+		d->session[i - skip] = d->session[i];
+	}
+	if (!skip)
+		return 0;
+	burn_session_free(s);
+	d->sessions--;
+	return 1;
+}
+
+
 struct burn_track *burn_track_create(void)
 {
 	struct burn_track *t;
@@ -330,11 +354,19 @@ void burn_track_clear_isrc(struct burn_track *t)
 int burn_track_get_sectors(struct burn_track *t)
 {
 	/* ts A70125 : was int */
-	off_t size;
+	off_t size = 0;
 	int sectors, seclen;
 
 	seclen = burn_sector_length(t->mode);
-	size = t->offset + t->source->get_size(t->source) + t->tail;
+	if (t->source != NULL)                /* ts A80808 : mending sigsegv */
+		size = t->offset + t->source->get_size(t->source) + t->tail;
+	else if(t->entry != NULL) {
+		/* ts A80808 : all burn_toc_entry of track starts should now
+			have (extensions_valid & 1), even those from CD.
+		*/
+		if (t->entry->extensions_valid & 1)
+			size = ((off_t) t->entry->track_blocks) * (off_t) 2048;
+	}
 	sectors = size / seclen;
 	if (size % seclen)
 		sectors++;
@@ -468,6 +500,7 @@ int burn_session_get_sectors(struct burn_session *s)
 	return sectors;
 }
 
+
 int burn_disc_get_sectors(struct burn_disc *d)
 {
 	int sectors = 0, i;
@@ -515,3 +548,84 @@ int burn_session_get_hidefirst(struct burn_session *session)
 {
 	return session->hidefirst;
 }
+
+
+/* ts A80808 : Enhance CD toc to DVD toc */
+int burn_disc_cd_toc_extensions(struct burn_disc *d, int flag)
+{
+	int sidx= 0, tidx= 0;
+	struct burn_toc_entry *entry, *prev_entry= NULL;
+	/* ts A81126 : ticket 146 : There was a SIGSEGV in here */
+	char msg_data[321], *msg;
+
+	strcpy(msg_data,
+		"Damaged CD table-of-content detected and truncated.");
+	strcat(msg_data, " In burn_disc_cd_toc_extensions: ");
+        msg = msg_data + strlen(msg_data);
+	if (d->session == NULL) {
+		strcpy(msg, "d->session == NULL");
+		goto failure;
+	}
+	for (sidx = 0; sidx < d->sessions; sidx++) {
+		if (d->session[sidx] == NULL) {
+			sprintf(msg, "d->session[%d of %d] == NULL",
+				sidx, d->sessions);
+			goto failure;
+		}
+		if (d->session[sidx]->track == NULL) {
+			sprintf(msg, "d->session[%d of %d]->track == NULL",
+				sidx, d->sessions);
+			goto failure;
+		}
+		if (d->session[sidx]->leadout_entry == NULL) {
+			sprintf(msg,
+				" Session %d of %d: Leadout entry missing.",
+			  	sidx, d->sessions);
+			goto failure;
+		}
+		for (tidx = 0; tidx < d->session[sidx]->tracks + 1; tidx++) {
+			if (tidx < d->session[sidx]->tracks) {
+				if (d->session[sidx]->track[tidx] == NULL) {
+					sprintf(msg,
+			  "d->session[%d of %d]->track[%d of %d] == NULL",
+			   sidx, d->sessions, tidx,  d->session[sidx]->tracks);
+					goto failure;
+				}
+				entry = d->session[sidx]->track[tidx]->entry;
+				if (entry == NULL) {
+					sprintf(msg,
+			  "session %d of %d, track %d of %d, entry == NULL",
+			  			sidx, d->sessions, tidx,
+						d->session[sidx]->tracks);
+					goto failure;
+				}
+			} else
+				entry = d->session[sidx]->leadout_entry;
+			entry->session_msb = 0;
+			entry->point_msb = 0;
+			entry->start_lba = burn_msf_to_lba(entry->pmin,
+						entry->psec, entry->pframe);
+			if (tidx > 0) {
+				prev_entry->track_blocks =
+					entry->start_lba
+					- prev_entry->start_lba;
+				prev_entry->extensions_valid |= 1;
+			}
+			if (tidx == d->session[sidx]->tracks) {
+				entry->session_msb = 0;
+				entry->point_msb = 0;
+				entry->track_blocks = 0;
+				entry->extensions_valid |= 1;
+			}
+			prev_entry = entry;
+		}
+	}
+	return 1;
+failure:
+	libdax_msgs_submit(libdax_messenger, -1, 0x0002015f,
+		LIBDAX_MSGS_SEV_MISHAP, LIBDAX_MSGS_PRIO_HIGH, msg_data, 0, 0);
+	d->sessions= sidx;
+	return 0;
+}
+
+
