@@ -171,6 +171,19 @@ struct burn_drive
 	/* Link Size item number 0 from feature 0021h descriptor */
 	int current_feat21h_link_size;
 
+	/* Flags from feature 0023h for formatting BD mmc5r03c.pdf 5.3.13 
+           Byte 4 BD-RE:
+             bit0= Cert   format 30h sub-type 10b
+             bit1= QCert  format 30h sub-type 11b
+             bit2= Expand format 01h
+             bit3= RENoSA format 31h
+           Byte 8 BD-R:
+             bit0= RRM    format 32h sub-type 10b
+        */
+	int current_feat23h_byte4;
+	int current_feat23h_byte8;
+
+
 	/* Flags from feature 002Fh feature descriptor mmc5r03c.pdf 5.3.25 :
 	     bit1= DVD-RW supported
 	     bit2= Test Write available
@@ -192,6 +205,11 @@ struct burn_drive
 	               with BD-RE */
 	int do_stream_recording;
 
+        /* ts A90227 : the LBA where stream recording shall start.
+                       Writing to lower LBA will be done without streaming.
+        */
+        int stream_recording_start;
+
 	/* ts A61218 from 51h READ DISC INFORMATION */
 	int bg_format_status; /* 0=needs format start, 1=needs format restart*/
 
@@ -223,6 +241,9 @@ struct burn_drive
 
 	/* ts A70131 : from 51h READ DISC INFORMATION Number of Sessions (-1)*/
 	int complete_sessions;
+	/* ts A90107 */
+	int state_of_last_session;
+
 	/* ts A70129 :
 	   from 51h READ DISC INFORMATION Last Track Number in Last Session */
 	int last_track_no;
@@ -235,6 +256,9 @@ struct burn_drive
 	/* ts A70215 : if > 0 : first lba on media that is too high for write*/
 	int media_lba_limit;
 
+	/* ts A81210 : Upper limit of readable data size,
+	               0x7fffffff = unknown */
+	int media_read_capacity;
 
 	int toc_temp;
 	struct burn_disc *disc;	/* disc structure */
