@@ -1005,8 +1005,10 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 	} else if (d->drive_role == 3 ||
 		   d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	           d->current_profile == 0x15 ||
-	           d->current_profile == 0x1b || d->current_profile == 0x2b ) {
-		/* DVD-R* Sequential , DVD+R[/DL] , sequential stdio "drive" */
+	           d->current_profile == 0x1b || d->current_profile == 0x2b ||
+		   d->current_profile == 0x41) {
+		/* DVD-R* Sequential , DVD+R[/DL] , BD-R,
+		   sequential stdio "drive" */
 		if (o->start_byte >= 0)
 			strcat(reasons, "write start address not supported, ");
 	} else {
@@ -1106,6 +1108,10 @@ int burn_disc_open_track_dvd_plus_r(struct burn_write_opts *o,
 	if (o->write_type == BURN_WRITE_SAO &&
 	    ! burn_track_is_open_ended(s->track[tnum])) {
  		/* Round track size up to 32 KiB and reserve track */
+
+		/* ts A81208 */
+		/* >>> ??? round to 64 KiB for BD-R ? (It is not mandatory) */
+
 		size = ((off_t) burn_track_get_sectors(s->track[tnum]))
 			 * (off_t) 2048;
 		size = (size + (off_t) 0x7fff) & ~((off_t) 0x7fff);
@@ -1153,19 +1159,29 @@ int burn_disc_close_track_dvd_minus_r(struct burn_write_opts *o,
 int burn_disc_finalize_dvd_plus_r(struct burn_write_opts *o)
 {
 	struct burn_drive *d = o->drive;
+	char msg[80];
 
+	sprintf(msg, "Finalizing %s ...",
+		d->current_profile_text);
 	libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00000002,
 			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
-			"Finalizing DVD+R ...", 0, 0);
+			msg, 0, 0);
 
-	/* CLOSE SESSION, 101b, Finalize with minimal radius */
-	d->close_track_session(d, 2, 1);  /* (2<<1)|1 = 5 */
+	if(d->current_profile == 0x41) { /* BD-R */
+		/* CLOSE SESSION, 110b, Finalize Disc */
+		d->close_track_session(d, 3, 0);  /* (3<<1)|0 = 6 */
+	} else {
+		/* CLOSE SESSION, 101b, Finalize with minimal radius */
+		d->close_track_session(d, 2, 1);  /* (2<<1)|1 = 5 */
+	}
 
+	sprintf(msg, "... finalizing %s done               ",
+		d->current_profile_text);
 	libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00000002,
 			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
-			"... finalizing DVD+R done               ", 0, 0);
+			msg, 0, 0);
 
 	return 1;
 }
@@ -1200,7 +1216,7 @@ int burn_disc_close_track_dvd_plus_r(struct burn_write_opts *o,
 }
 
 
-/* ts A61218 - A70129 */
+/* ts A61218 - A81208 */
 int burn_dvd_write_track(struct burn_write_opts *o,
 			struct burn_session *s, int tnum, int is_last_track)
 {
@@ -1224,6 +1240,11 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 		ret = burn_disc_open_track_dvd_plus_r(o, s, tnum);
 		if (ret <= 0)
 			goto ex;
+	} else if (d->current_profile == 0x41) {
+		/* BD-R SRM */
+		ret = burn_disc_open_track_dvd_plus_r(o, s, tnum);
+		if (ret <= 0)
+			goto ex;
 	}
 
 	sectors = burn_track_get_sectors(t);
@@ -1292,6 +1313,12 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 							 is_last_track);
 		if (ret <= 0)
 			goto ex;
+	} else if (d->current_profile == 0x41) {
+		/* BD-R SRM */
+		ret = burn_disc_close_track_dvd_plus_r(o, s, tnum,
+							 is_last_track);
+		if (ret <= 0)
+			goto ex;
 	}
 	ret = 1;
 ex:;
@@ -1372,10 +1399,70 @@ int burn_disc_close_session_dvd_minus_r(struct burn_write_opts *o,
 int burn_dvd_write_session(struct burn_write_opts *o,
 				struct burn_session *s, int is_last_session)
 {
-	int i,ret;
+	int i, ret, multi_mem;
         struct burn_drive *d = o->drive;
 
-	/* >>> open_session ? */
+	/* ts A90108 */
+	if (d->current_profile == 0x41 && d->status == BURN_DISC_APPENDABLE &&
+	    d->state_of_last_session == 1) {
+		/* last session on BD-R is still open */;
+
+		/* BR-R were not closed by libburn-0.6.0.pl00 if o->multi==0.
+		   This leads to an unreadable, but recoverable) media state.
+		   Technically they are appendable although the last session
+		   is not readable.
+
+		   By default the open session gets closed here before the new
+		   session is written. E.g. after writing a small dummy seesion
+		   number 2 one can read session 1 and write session 3 which
+		   points to data of session 1.
+
+		   For the case that no media with 3 sessions is desired it is
+		   possible to activate the following coarse single-session
+		   closing code:
+		   No new session will be written but calling programs will
+		   report success. Quite misleading.
+		   Activate only if really needed by
+		   # define Libburn_bug_A90108_close_disC yes
+		*/
+
+
+#ifdef Libburn_bug_A90108_close_disC
+
+		/* Close open session and media. 
+		   That was the goal of the failed run which led to the
+		   unreadable (but recoverable) media state.
+
+		   It is not easy to implement a general close function for
+		   all media types. Therefore this pseudo write code is under
+		   control of #ifdef.
+		*/
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x00020171,
+				LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
+				"Closing BD-R with accidently open session",
+				0, 0);
+ 		d->close_track_session(d, 3, 0); /* CLOSE SESSION, 110b */
+		d->state_of_last_session = 3; /* mark as complete session */
+		d->status = BURN_DISC_FULL;
+		sleep(3); /* The caller might need time to arrange itself */
+		return 1;
+
+#else /* Libburn_bug_A90108_close_disC */
+
+		/* This is the default mode.
+		*/
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x00020170,
+				LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
+				"Closing open session before writing new one",
+				0, 0);
+ 		d->close_track_session(d, 1, 0); /* CLOSE SESSION, 010b */
+		d->state_of_last_session = 3; /* mark as complete session */
+
+#endif /* ! Libburn_bug_A90108_close_disC */
+
+	}
 
 	for (i = 0; i < s->tracks; i++) {
 		ret = burn_dvd_write_track(o, s, i,
@@ -1386,7 +1473,11 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 	if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	    d->current_profile == 0x15) {
 		/* DVD-R , DVD-RW Sequential, DVD-R/DL Sequential */
+		multi_mem = o->multi;
+		if (!is_last_session)
+			o->multi = 1;
 		ret = burn_disc_close_session_dvd_minus_r(o, s);
+		o->multi = multi_mem;
 		if (ret <= 0)
 			return 0;
 	} else if (d->current_profile == 0x12 || d->current_profile == 0x43) {
@@ -1408,6 +1499,8 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 		}
 	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b) {
 		/* DVD+R , DVD+R/DL do each track as an own session */;
+	} else if (d->current_profile == 0x41) {
+		/* BD-R SRM do each track as an own session */;
 	}
 	return 1;
 }
@@ -1611,8 +1704,12 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 		/* ??? padding needed ??? cowardly doing it for now */
 		o->obs_pad = 1; /* fill-up track's last 32k buffer */
 		
-	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b) {
-		/* DVD+R , DVD+R/DL */
+	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b ||
+		   d->current_profile == 0x41) {
+		/* DVD+R , DVD+R/DL , BD-R SRM */
+
+		/* >>> ts A81208 : with BD-R set o->obs to 64 kB ? */
+
 		t = disc->session[0]->track[0];
 		o_end = ( burn_track_is_open_ended(t) && !o->fill_up_media );
 		default_size = burn_track_get_default_size(t);
@@ -1627,8 +1724,8 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 		}
 		ret = burn_disc_setup_dvd_plus_r(o, disc);
 		if (ret <= 0) {
-			sprintf(msg,
-			  "Write preparation setup failed for DVD+R");
+			sprintf(msg, "Write preparation setup failed for %s",
+			   	d->current_profile == 0x41 ? "BD-R" : "DVD+R");
 			libdax_msgs_submit(libdax_messenger, d->global_index,
 				0x00020121,
 				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
@@ -1996,9 +2093,9 @@ void burn_disc_write_sync(struct burn_write_opts *o, struct burn_disc *disc)
 {
 	struct cue_sheet *sheet;
 	struct burn_drive *d = o->drive;
-	struct buffer buf;
+	struct buffer buf, *buffer_mem = o->drive->buffer;
 	struct burn_track *lt, *t;
-	int first = 1, i, ret, lba, nwa = 0;
+	int first = 1, i, ret, lba, nwa = 0, multi_mem;
 	off_t default_size;
 	char msg[80];
 
@@ -2009,8 +2106,12 @@ void burn_disc_write_sync(struct burn_write_opts *o, struct burn_disc *disc)
 	/* ts A61224 */
 	burn_disc_init_write_status(o, disc); /* must be done very early */
 
-	/* ts A80412 */
-	d->do_stream_recording = o->do_stream_recording;
+	/* ts A80412 , A90227 */
+	d->do_stream_recording = !!o->do_stream_recording;
+	if (o->do_stream_recording >= 16)
+		d->stream_recording_start = o->do_stream_recording;
+	else
+		d->stream_recording_start = 0;
 
 	d->buffer = &buf;
 	memset(d->buffer, 0, sizeof(struct buffer));
@@ -2148,7 +2249,12 @@ return crap.  so we send the command, then ignore the result.
 				d->alba += 4500;
 			}
 		}
-		if (!burn_write_session(o, disc->session[i]))
+		multi_mem = o->multi;
+		if(i < disc->sessions - 1)
+			o->multi = 1;
+		ret = burn_write_session(o, disc->session[i]);
+		o->multi = multi_mem;
+		if (!ret)
 			goto fail;
 
 		lt = disc->session[i]->track[disc->session[i]->tracks - 1];
@@ -2206,6 +2312,7 @@ fail_wo_sync:;
 	d->busy = BURN_DRIVE_IDLE;
 ex:;
 	d->do_stream_recording = 0;
+	d->buffer = buffer_mem;
 	return;
 }
 
@@ -2215,7 +2322,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 {
 	int alignment = 0, start, upto, chunksize, err, fd = -1, ret;
 	char msg[81], *rpt;
-	struct buffer buf;
+	struct buffer buf, *buffer_mem = d->buffer;
 
 	if (d->released) {
 		libdax_msgs_submit(libdax_messenger,
@@ -2326,7 +2433,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		
 	if(fd >= 0)
 		close(fd);
-	d->buffer = NULL;
+	d->buffer = buffer_mem;
 	d->busy = BURN_DRIVE_IDLE;
 	return 1;
 }
